<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;
use Joomla\CMS\Installer\InstallerAdapter;
use Joomla\Database\DatabaseInterface;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * VP Advanced Custom Field Component Package Script
 *
 * @since   1.0
 */
// phpcs:ignore
class Pkg_vmcustomadvancedInstallerScript
{
    /**
     * Called on installation
     *
     * @param   InstallerAdapter  $adapter  The object responsible for running this script
     *
     * @return  boolean  True on success
     */
    public function install($adapter)
    {
        if ($this->enablePlugins()) {
            Factory::getApplication()->enqueueMessage('VP Advanced Custom Field Component package has been successfully installed in your site.');
        }

        return true;
    }

    /**
     * Called on update
     *
     * @param   InstallerAdapter  $adapter  The object responsible for running this script
     *
     * @return  boolean  True on success
     */
    public function update($adapter)
    {
        if ($this->enablePlugins()) {
            Factory::getApplication()->enqueueMessage('VP Advanced Custom Field Component package has been successfully updated in your site.');
        }

        return true;
    }

    /**
    * Private method to enable all required plugins of the component
    *
    * @return boolean
    */
    private function enablePlugins()
    {
        $app     = Factory::getApplication();
        $plugins = $this->getPluginIDs();

        if ($plugins === false || !is_array($plugins)) {
            $app->enqueueMessage('Please go to Plugin Manager and enable the two "VP Advanced Custom Field" plugins.');
            return false;
        }

        if (count($plugins) !== 2) {
            $app->enqueueMessage('Package not installed correctly. Missing required plugins.');
            return false;
        }

        $hasError = false;

        if (version_compare(JVERSION, '4.0.0', 'ge')) {
            /** @var \Joomla\Component\Installer\Administrator\Extension\InstallerComponent $component **/
            $component = Factory::getApplication()->bootComponent('plugins');
            $factory   = $component->getMVCFactory();

            /** @var \Joomla\Component\Plugins\Administrator\Model\PluginModel $model */
            $model = $factory->createModel('Plugin', 'Administrator', ['ignore_requests' => true]);

            foreach ($plugins as $extension_id) {
                if (!$model->publish($extension_id, 1)) {
                    $hasError = true;
                }
            }
        } else {
            jimport('legacy.model.legacy');
            JModelLegacy::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_plugins/models/', 'PluginsModel');

            /** @var \Joomla\Component\Plugins\Administrator\Model\PluginModel $model */
            $model = JModelLegacy::getInstance('Plugin', 'PluginsModel');

            if (!$model) {
                $app->enqueueMessage('Please go to Plugin Manager and enable the two "VP Advanced Custom Field" plugins.');
                return false;
            }

            foreach ($plugins as $extension_id) {
                if (!$model->publish($extension_id, 1)) {
                    $hasError = true;
                }
            }
        }

        if ($hasError) {
            $app->enqueueMessage('Please go to Plugin Manager and enable the two "VP Advanced Custom Field" plugins.');
            return false;
        }

        return true;
    }

    /**
    * Private method to get the installed VP Advanced Custom Field plugin IDs.
    *
    * @return mixed(boolean/array)  false if not found.
    */
    private function getPluginIDs()
    {
        $db    = version_compare(JVERSION, '4.0.0', 'ge') ? Factory::getContainer()->get(DatabaseInterface::class) : Factory::getDBO();
        $query = $db->getQuery(true);

        $query->select($db->quoteName('extension_id'))
            ->from($db->quoteName('#__extensions'))
            ->where($db->quoteName('type') . ' = ' . $db->quote('plugin'))
            ->where($db->quoteName('element') . ' = ' . $db->quote('vmcustomadvanced'))
            ->where($db->quoteName('folder') . ' = ' . $db->quote('system'));

        try {
            $db->setQuery($query);
            $system_plugin_id = $db->loadResult();
        } catch (Exception $e) {
            // We can not throw SQL error here.
            return false;
        }

        $query->clear()
            ->select($db->quoteName('extension_id'))
            ->from($db->quoteName('#__extensions'))
            ->where($db->quoteName('type') . ' = ' . $db->quote('plugin'))
            ->where($db->quoteName('element') . ' = ' . $db->quote('advanced'))
            ->where($db->quoteName('folder') . ' = ' . $db->quote('vmcustom'));

        try {
            $db->setQuery($query);
            $vmcustom_plugin_id = $db->loadResult();
        } catch (Exception $e) {
            // We can not throw SQL error here.
            return false;
        }

        $extension_ids = [];

        if (!empty($system_plugin_id)) {
            $extension_ids[] = $system_plugin_id;
        }

        if (!empty($vmcustom_plugin_id)) {
            $extension_ids[] = $vmcustom_plugin_id;
        }

        return $extension_ids;
    }
}
