<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;
use Joomla\String\StringHelper;
use Joomla\CMS\Language\Language;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * VMCustomAdvanced helper.
 */
class VMCustomAdvancedHelper
{
    protected static $extensions;

    /**
     * Configure the Linkbar.
     *
     * @param   string  $vName  string
     *
     * @return void
     */
    public static function addSubmenu($vName = '')
    {
        JHtmlSidebar::addEntry(JText::_('COM_VMCUSTOMADVANCED_TITLE_SETS'), 'index.php?option=com_vmcustomadvanced&view=sets', $vName == 'sets');
        JHtmlSidebar::addEntry(JText::_('COM_VMCUSTOMADVANCED_TITLE_OPTIONS'), 'index.php?option=com_vmcustomadvanced&view=options', $vName == 'options');
        JHtmlSidebar::addEntry(JText::_('COM_VIRTUEMART_MENU_CUSTOMS'), 'index.php?option=com_virtuemart&view=custom', $vName == 'custom');
        JHtmlSidebar::addEntry(JText::_('COM_VIRTUEMART_MENU_PRODUCTS'), 'index.php?option=com_virtuemart&view=product', $vName == 'product');
    }

    public static function registerVirtueMart()
    {
        // Register VirtueMart config to Joomla autoloader
        JLoader::register('VmConfig', JPATH_ADMINISTRATOR . '/components/com_virtuemart/helpers/config.php');
        JLoader::register('vmDefines', JPATH_ADMINISTRATOR . '/components/com_virtuemart/helpers/vmdefines.php');

        // Load VirtueMart Config and language
        VmConfig::loadConfig();
        VmConfig::loadJLang('com_virtuemart');
        VmConfig::loadJLang('com_virtuemart.sys');

        // Register all required VirtueMart classes to Joomla autoloader
        JLoader::register('vmVersion', JPATH_ADMINISTRATOR . '/components/com_virtuemart/version.php');
        JLoader::register('VirtueMartModelVendor', JPATH_ADMINISTRATOR . '/components/com_virtuemart/models/vendor.php');
        JLoader::register('VmModel', JPATH_ADMINISTRATOR . '/components/com_virtuemart/helpers/vmmodel.php');
        JLoader::register('VmTable', JPATH_ADMINISTRATOR . '/components/com_virtuemart/helpers/vmtable.php');
        JLoader::register('calculationHelper', JPATH_ADMINISTRATOR . '/components/com_virtuemart/helpers/calculationh.php');
        JLoader::register('ShopFunctions', JPATH_ADMINISTRATOR . '/components/com_virtuemart/helpers/shopfunctions.php');
        JLoader::register('VmImage', JPATH_ADMINISTRATOR . '/components/com_virtuemart/helpers/image.php');
        JLoader::register('TableMedias', JPATH_ADMINISTRATOR . '/components/com_virtuemart/tables/medias.php');
        JLoader::register('TableCustoms', JPATH_ADMINISTRATOR . '/components/com_virtuemart/tables/customs.php');
        JLoader::register('TableCategories', JPATH_ADMINISTRATOR . '/components/com_virtuemart/tables/categories.php');
        JLoader::register('VirtueMartModelCustom', JPATH_ADMINISTRATOR . '/components/com_virtuemart/models/custom.php');
        JLoader::register('VirtueMartModelCategory', JPATH_ADMINISTRATOR . '/components/com_virtuemart/models/category.php');
        JLoader::register('VirtueMartModelProduct', JPATH_ADMINISTRATOR . '/components/com_virtuemart/models/product.php');
        JLoader::register('vmCustomPlugin', JPATH_ADMINISTRATOR . '/components/com_virtuemart/plugins/vmcustomplugin.php');
    }

    /**
     * Gets a list of the actions that can be performed.
     *
     * @return    JObject
     */
    public static function getActions()
    {
        $user   = Factory::getUser();
        $result = new JObject();

        $assetName = 'com_vmcustomadvanced';

        $actions = array(
            'core.admin', 'core.manage', 'core.create', 'core.edit', 'core.edit.own', 'core.edit.state', 'core.delete'
        );

        foreach ($actions as $action) {
            $result->set($action, $user->authorise($action, $assetName));
        }

        return $result;
    }

    public static function stringLangKeySafe($string, $language = '')
    {
        // Remove any '-' from the string since they will be used as concatenaters
        $str = str_replace('_', ' ', $string);

        // Transliterate on the language requested (fallback to current language if not specified)
        $lang = $language == '' || $language == '*' ? Factory::getLanguage() : Language::getInstance($language);
        $str = $lang->transliterate($str);

        // Trim white spaces at beginning and end of alias and make lowercase
        $str = trim(StringHelper::strtoupper($str));

        // Remove any duplicate whitespace, and ensure all characters are alphanumeric
        $str = preg_replace('/(\s|[^A-Za-z0-9\-])+/', '_', $str);

        // Trim dashes at beginning and end of alias
        $str = trim($str, '_');

        return $str;
    }

    public static function buildLanguageTranslation($section = 'sets')
    {
        $section = strtolower(trim($section));

        if (!in_array($section, ['sets', 'options'])) {
            return false;
        }

        $db    = Factory::getDbo();
        $query = $db->getQuery(true);

        if ($section == 'sets') {
            $query->select($db->quoteName('title'))
                ->select($db->quoteName('alias'))
                ->from($db->quoteName('#__vmcustomadvanced_sets'));

            $db->setQuery($query);
            $languages = $db->loadObjectList();
        } else {
            $query->select($db->quoteName('title'))
                ->select($db->quoteName('alias'))
                ->from($db->quoteName('#__vmcustomadvanced_options'));

            $db->setQuery($query);
            $languages = $db->loadObjectList();
        }

        if (empty($languages)) {
            return true;
        }

        $strings = [];

        foreach ($languages as $language) {
            $strings = [$language->alias => $language->title] + $strings;
        }

        if (!empty($strings)) {
            return self::saveLanguageTranslation($strings, $section);
        }

        return true;
    }

    public static function saveLanguageTranslation($strings, $section, $admin = false)
    {
        $default_language = Factory::getLanguage()->getDefault();
        $current_language = Factory::getLanguage()->getTag();

        $files = [];

        $files[] = ($admin ? JPATH_ADMINISTRATOR : JPATH_SITE) . '/language/en-GB/en-GB.com_vmcustomadvanced_' . $section . '.ini';

        if ($default_language != 'en-GB' && !empty($default_language)) {
            $files[] = ($admin ? JPATH_ADMINISTRATOR : JPATH_SITE) . '/language/' . $default_language . '/' . $default_language . '.com_vmcustomadvanced_' . $section . '.ini';
        }

        if ($current_language != $default_language && $current_language != 'en-GB' && !empty($current_language)) {
            $files[] = ($admin ? JPATH_ADMINISTRATOR : JPATH_SITE) . '/language/' . $current_language . '/' . $current_language . '.com_vmcustomadvanced_' . $section . '.ini';
        }

        foreach ($files as $fileName) {
            if (JLanguageHelper::saveToIniFile($fileName, $strings) === false) {
                return false;
            }
        }

        if (!$admin) {
            return self::saveLanguageTranslation($strings, $section, true);
        }

        return true;
    }

    public static function checkSystem()
    {
        $app = Factory::getApplication();

        $systemPluginEnabled   = self::hasExtension('vmcustomadvanced', 'plugin', 'system');
        $vmcustomPluginEnabled = self::hasExtension('advanced', 'plugin', 'vmcustom');

        if (!$systemPluginEnabled || !$vmcustomPluginEnabled) {
            $message = '<p>' . JText::_('COM_VMCUSTOMADVANCED_WARNING_REQUIRED_PLUGINS_MISSING') . '</p>';

            $url  = JRoute::_('index.php?option=com_plugins&view=plugins&filter_search=vp+advanced+custom+field');
            $html = array();

            if (!$systemPluginEnabled) {
                $html[] = '<a href="' . $url . '" target="_blank">System - VP Advanced Custom Field</a>';
            }

            if (!$vmcustomPluginEnabled) {
                $html[] = '<a href="' . $url . '" target="_blank">VM Custom - VP Advanced Custom Field</a>';
            }

            if (!empty($html)) {
                $message .= '<ol><li>' . implode('</li><li>', $html) . '</li></ol>';
            }

            $app->enqueueMessage($message, 'warning');
        }

        if (self::hasExtension('pkg_vmcustomadvanced', 'package')) {
            $params      = JComponentHelper::getParams('com_vmcustomadvanced');
            $downloadKey = $params->get('download_key', '');
            $downloadKey = $downloadKey ? trim($downloadKey) : null;

            if (empty($downloadKey)) {
                $app->enqueueMessage(JText::_('COM_VMCUSTOMADVANCED_WARNING_UPDATE_DOWNLOAD_KEY'), 'info');
            }
        }
    }

    /**
    * Method to check if an extension installed and enabled
    *
    * @param  string  $element  Name of the extension
    * @param  string  $type     Type of the extension like component, module, plugins etc
    *
    * @return boolean
    */
    public static function hasExtension($element, $type = 'component', $folder = null)
    {
        if (!is_array(self::$extensions)) {
            self::$extensions = array();
        }

        $hash = 'element:' . $element . '.type:' . $type;

        if (!empty($folder)) {
            $hash .= '.folder:' . $folder;
        }

        if (!array_key_exists($hash, self::$extensions)) {
            $type = strtolower($type);
            self::$extensions[$hash] = false;

            $db    = Factory::getDbo();
            $query = $db->getQuery(true)
                ->select($db->quoteName('extension_id'))
                ->select($db->quoteName('element'))
                ->select($db->quoteName('folder'))
                ->from($db->quoteName('#__extensions'))
                ->where($db->quoteName('type') . ' = ' . $db->quote($type))
                ->where($db->quoteName('element') . ' = ' . $db->quote($element))
                ->where($db->quoteName('enabled') . ' = 1');

            try {
                $db->setQuery($query);
                $results = $db->loadObjectList();
            } catch (Exception $e) {
                Factory::getApplication()->enqueueMessage($e->getMessage(), 'error');
                return false;
            }

            if (!empty($results)) {
                foreach ($results as $result) {
                    if ($type != 'package') {
                        switch ($type) {
                            case 'module':
                                $path = JPATH_SITE . '/modules/' . $result->element;
                                break;
                            case 'plugin':
                                if (!empty($folder) && strtolower($folder) != $result->folder) {
                                    $path = null;
                                } else {
                                    $path = JPATH_SITE . '/plugins/' . $result->folder . '/' . $result->element;
                                }
                                break;
                            default:
                            case 'component':
                                $path = JPATH_ADMINISTRATOR . '/components/' . $result->element;
                        }

                        if ($path && file_exists($path) && is_dir($path)) {
                            self::$extensions[$hash] = true;
                            break;
                        }
                    } else {
                        self::$extensions[$hash] = true;
                        break;
                    }
                }
            }
        }

        return self::$extensions[$hash];
    }
}
