<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;

if (!trait_exists('VmcustomadvancedModelHistoryTrait')) {
    require __DIR__ . '/historytrait.php';
}
// phpcs:enable PSR1.Files.SideEffects

use Joomla\CMS\Factory;
use Joomla\Registry\Registry;

/**
 * VMCustomAdvancedOption model.
 */
class VMCustomAdvancedModelOption extends JModelAdmin
{
    use VmcustomadvancedModelHistoryTrait;

    /**
     * @var  string    The prefix to use with controller messages.
     */
    protected $text_prefix = 'COM_VMCUSTOMADVANCED';

    /**
     * @var  string    Alias to manage history control
     */
    public $typeAlias = 'com_vmcustomadvanced.option';

    /**
     * @var  null      Item data
     */
    protected $item = null;

    /**
     * Returns a reference to the a Table object, always creating it.
     *
     * @param   string  $type    The table type to instantiate
     * @param   string  $prefix  A prefix for the table class name. Optional.
     * @param   array   $config  Configuration array for model. Optional.
     *
     * @return    JTable    A database object
     */
    public function getTable($type = 'Option', $prefix = 'VMCustomAdvancedTable', $config = [])
    {
        return JTable::getInstance($type, $prefix, $config);
    }

    /**
     * Method to get the record form.
     *
     * @param   array    $data      An optional array of data for the form to interogate.
     * @param   boolean  $loadData  True if the form is to load its own data (default case), false if not.
     *
     * @return  JForm  A JForm object on success, false on failure
     */
    public function getForm($data = [], $loadData = true)
    {
        // Get the form.
        $form = $this->loadForm('com_vmcustomadvanced.option', 'option', ['control' => 'jform', 'load_data' => $loadData]);

        if (empty($form)) {
            return false;
        }

        return $form;
    }

    /**
     * Method to get the data that should be injected in the form.
     *
     * @return   mixed  The data for the form.
     */
    protected function loadFormData()
    {
        /** @var \Joomla\CMS\Application\CMSApplication */
        $app = Factory::getApplication();

        // Check the session for previously entered form data.
        $data = $app->getUserState('com_vmcustomadvanced.edit.option.data', []);

        if (empty($data)) {
            if ($this->item === null) {
                $this->item = $this->getItem();
            }

            $data = $this->item;
        }

        return $data;
    }

    /**
     * Override JModelAdmin::preprocessForm to ensure the correct plugin group is loaded.
     *
     * @param   JForm   $form   A JForm object.
     * @param   mixed   $data   The data expected for the form.
     * @param   string  $group  The name of the plugin group to import (defaults to "content").
     *
     * @return  void
     *
     * @throws  Exception if there is an error in the form event.
     */
    protected function preprocessForm($form, $data, $group = 'content')
    {
        if (!empty($data)) {
            $set_id = 0;

            if (is_object($data)) {
                $set_id = (int) $data->set_id;
            } elseif (is_array($data)) {
                $set_id = (int) $data['set_id'];
            }

            if (!empty($set_id)) {
                $db    = $this->getDbo();
                $query = $db->getQuery(true);

                $query->select($db->quoteName('type'))
                    ->select($db->quoteName('option_type'))
                    ->select($db->quoteName('params'))
                    ->from($db->quoteName('#__vmcustomadvanced_sets'))
                    ->where($db->quoteName('id') . ' = ' . $set_id);

                $db->setQuery($query);
                $set = $db->loadObject();

                if (!empty($set)) {
                    $set->params = new Registry($set->params);
                }
            }

            if (!empty($set)) {
                if (empty($set->option_type)) {
                    $form->removeField('color');
                    $form->removeField('image');
                } elseif ($set->option_type == 1) {
                    $form->removeField('image');
                    $form->setFieldAttribute('color', 'required', true);
                } elseif ($set->option_type == 2) {
                    $form->removeField('color');
                    $form->setFieldAttribute('image', 'required', true);
                }

                if ($set->params->get('is_button', 1) && $set->type == 1 && !empty($set->option_type)) {
                    $form->setFieldAttribute('tick_color', 'required', true, 'params');
                } else {
                    $form->removeField('tick_color', 'params');
                }
            }
        }

        if (version_compare(JVERSION, '4.0.0', 'ge')) {
            $form->setFieldAttribute('state', 'class', '');
            $form->setFieldAttribute('state', 'layout', 'joomla.form.field.radio.switcher');
        }

        parent::preprocessForm($form, $data, $group);
    }

    /**
     * Method to get a single record.
     *
     * @param   integer  $pk  The id of the primary key.
     *
     * @return  mixed    Object on success, false on failure.
     */
    public function getItem($pk = null)
    {
        if ($item = parent::getItem($pk)) {
            // Do any procesing on fields here if needed
        }

        return $item;
    }

    /**
     * Method to duplicate an Option
     *
     * @param   array  &$pks  An array of primary key IDs.
     *
     * @return  boolean  True if successful.
     *
     * @throws  Exception
     */
    public function duplicate(&$pks)
    {
        $user = Factory::getUser();

        // Access checks.
        if (!$user->authorise('core.create', 'com_vmcustomadvanced')) {
            throw new Exception(JText::_('JERROR_CORE_CREATE_NOT_PERMITTED'));
        }

        $table = $this->getTable();

        foreach ($pks as $pk) {
            if ($table->load($pk, true)) {
                // Reset the id to create a new record.
                $table->id = 0;

                if (!$table->check()) {
                    throw new Exception($table->getError());
                }
            } else {
                throw new Exception($table->getError());
            }
        }

        if (VMCustomAdvancedHelper::buildLanguageTranslation('options') === false) {
            $this->setError('Failed to create language translation files.');

            return false;
        }

        // Clean cache
        $this->cleanCache();

        return true;
    }

    /**
     * Method to save the form data.
     *
     * @param   array  $data  The form data.
     *
     * @return  boolean  True on success, False on error.
     */
    public function save($data)
    {
        $result = parent::save($data);

        if ($result) {
            if (VMCustomAdvancedHelper::buildLanguageTranslation('options') === false) {
                $this->setError('Failed to create language translation files.');

                return false;
            }
        }

        return $result;
    }
}
