<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * Methods supporting a list of VMCustomAdvanced records.
 */
class VMCustomAdvancedModelOptions extends JModelList
{
    /**
        * Constructor.
        *
        * @param   array  $config  An optional associative array of configuration settings.
        *
        * @see     JController
        */
    public function __construct($config = [])
    {
        if (empty($config['filter_fields'])) {
            $config['filter_fields'] = [
                'id', 'a.id',
                'ordering', 'a.ordering',
                'state', 'a.state',
                'title', 'a.title',
                'set_id', 'a.set_id',
                'set_title', 's.title',
                'created_by', 'a.created_by',
                'created_on', 'a.created_on'
            ];
        }

        parent::__construct($config);
    }

    /**
     * Method to auto-populate the model state.
     *
     * Note. Calling getState in this method will result in recursion.
     *
     * @param   string  $ordering   Elements order
     * @param   string  $direction  Order direction
     *
     * @return void
     *
     * @throws Exception
     */
    protected function populateState($ordering = 'a.id', $direction = 'ASC')
    {
        // List state information.
        parent::populateState($ordering, $direction);

        $context = $this->getUserStateFromRequest($this->context . '.context', 'context');
        $this->setState('filter.context', $context);
    }

    /**
     * Method to get a store id based on model configuration state.
     *
     * This is necessary because the model is used by the component and
     * different modules that might need different sets of data or different
     * ordering requirements.
     *
     * @param   string  $id  A prefix for the store id.
     *
     * @return   string A store id.
     */
    protected function getStoreId($id = '')
    {
        // Compile the store id.
        $id .= ':' . $this->getState('filter.search');
        $id .= ':' . $this->getState('filter.state');
        $id .= ':' . $this->getState('filter.set_id');

        return parent::getStoreId($id);
    }

    /**
     * Build an SQL query to load the list data.
     *
     * @return   JDatabaseQuery
     */
    protected function getListQuery()
    {
        // Create a new query object.
        $db    = $this->getDbo();
        $query = $db->getQuery(true);

        // Select the required fields from the table.
        $query->select($this->getState('list.select', 'DISTINCT a.*'));

        $query->from($db->quoteName('#__vmcustomadvanced_options') . ' AS a');

        // Join over the users for the checked out user
        $query->select('uc.' . $db->quoteName('name') . ' AS uEditor')
            ->join('LEFT', $db->quoteName('#__users') . ' AS uc ON uc.' . $db->quoteName('id') . ' = a.' . $db->quoteName('checked_out'));

        // Join over the user field 'created_by'
        $query->select('u.' . $db->quoteName('name') . ' AS created_by')
            ->join('LEFT', $db->quoteName('#__users') . ' AS u ON u.' . $db->quoteName('id') . ' = a.' . $db->quoteName('created_by'));

        // Join over the sets for 'set_title'
        $query->select('s.' . $db->quoteName('title') . ' AS set_title')
            ->select('s.' . $db->quoteName('option_type'))
            ->join('INNER', $db->quoteName('#__vmcustomadvanced_sets') . ' AS s ON s.' . $db->quoteName('id') . ' = a.' . $db->quoteName('set_id'));

        // Filter by published state
        $published = $this->getState('filter.state');

        if (is_numeric($published)) {
            $query->where('a.' . $db->quoteName('state') . ' = ' . (int) $published);
        } elseif ($published === '') {
            $query->where('a.' . $db->quoteName('state') . ' IN (0, 1)');
        }

        // Filter by search in title
        $search = $this->getState('filter.search');

        if (!empty($search)) {
            if (stripos($search, 'id:') === 0) {
                $query->where('a.' . $db->quoteName('id') . ' = ' . (int) substr($search, 3));
            } else {
                $search = $db->Quote('%' . $db->escape($search, true) . '%');
                $query->where('a.' . $db->quoteName('title') . ' LIKE ' . $search);
            }
        }

        // Filtering type
        $filter_set_id = $this->state->get('filter.set_id');

        if ($filter_set_id !== null && is_numeric($filter_set_id)) {
            $query->where('a.' . $db->quoteName('set_id') . ' = ' . (int) $filter_set_id);
        }

        // Add the list ordering clause.
        $orderCol  = $this->state->get('list.ordering', 'a.id');
        $orderDirn = $this->state->get('list.direction', 'ASC');

        if ($orderCol && $orderDirn) {
            $query->order($db->escape($orderCol . ' ' . $orderDirn));
        }

        return $query;
    }

    /**
     * Get an array of data items
     *
     * @return mixed Array of data items on success, false on failure.
     */
    public function getItems()
    {
        $items = parent::getItems();

        return $items;
    }
}
