<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Table\Table;
use Joomla\Registry\Registry;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;

JLoader::register('VmcustomadvancedTableHistoryInterface', __DIR__ . '/historyinterface.php');
JLoader::register('VMCustomAdvancedHelper', JPATH_ADMINISTRATOR . '/components/com_vmcustomadvanced/helpers/vmcustomadvanced.php');
// phpcs:enable PSR1.Files.SideEffects

/**
 * option Table class
 */
class VmcustomadvancedTableOption extends Table implements VmcustomadvancedTableHistoryInterface
{
    /**
     * Constructor
     *
     * @param   Joomla\Database\DatabaseDriver  &$db  A database connector object
     */
    public function __construct(&$db)
    {
        if (version_compare(JVERSION, '4.0.0', 'ge')) {
            $this->typeAlias = 'com_vmcustomadvanced.option';
        } else {
            JObserverMapper::addObserverClassToClass('JTableObserverContenthistory', 'VmcustomadvancedTableOption', ['typeAlias' => 'com_vmcustomadvanced.option']);
        }

        parent::__construct('#__vmcustomadvanced_options', 'id', $db);

        $this->setColumnAlias('published', 'state');
    }

    /**
     * Overloaded bind function to pre-process the params.
     *
     * @param   array  $array   Named array
     * @param   mixed  $ignore  Optional array or list of parameters to ignore
     *
     * @return  null|string  null is operation was satisfactory, otherwise returns an error
     */
    public function bind($array, $ignore = '')
    {
        $app   = Factory::getApplication();
        $input = $app->input;
        $task  = $input->input->get('task');

        if (empty($array['alias'])) {
            $array['alias'] = 'COM_VMCUSTOMADVANCED_OPTION_TITLE_' . $array['title'];
        }

        $array['alias'] = VMCustomAdvancedHelper::stringLangKeySafe(trim($array['alias']));

        if (isset($array['params']) && is_array($array['params'])) {
            $registry = new Registry();
            $registry->loadArray($array['params']);
            $array['params'] = (string) $registry;
        }

        if (!Factory::getUser()->authorise('core.admin', 'com_vmcustomadvanced.option.' . $array['id'])) {
            $actions         = JAccess::getActionsFromFile(JPATH_ADMINISTRATOR . '/components/com_vmcustomadvanced/access.xml', "/access/section[@name='option']/");
            $default_actions = JAccess::getAssetRules('com_vmcustomadvanced.option.' . $array['id'])->getData();
            $array_jaccess   = [];

            foreach ($actions as $action) {
                if (key_exists($action->name, $default_actions)) {
                    $array_jaccess[$action->name] = $default_actions[$action->name];
                }
            }

            $array['rules'] = $this->JAccessRulestoArray($array_jaccess);
        }

        // Bind the rules for ACL where supported.
        if (isset($array['rules']) && is_array($array['rules'])) {
            $this->setRules($array['rules']);
        }

        return parent::bind($array, $ignore);
    }

    /**
     * Overloaded check function
     *
     * @return bool
     */
    public function check()
    {
        $app   = Factory::getApplication();
        $input = $app->input;
        $task  = $input->input->get('task');

        if ($this->id == 0) {
            $this->ordering = $this->getNextOrder();
        }

        // Check if title is unique
        if (!$this->isUnique('title')) {
            if ($task == 'duplicate' || $task == 'save2copy') {
                $count = 0;
                $currentTitle =  $this->title;

                while (!$this->isUnique('title')) {
                    $this->title = $currentTitle . ' ' . $count++;
                }
            } else {
                $setTitle = $this->getSetTitle($this->set_id);

                throw new Exception(Text::sprintf('COM_VMCUSTOMADVANCED_ERROR_OPTION_TITLE_EXISTS', $this->title, $setTitle));
            }
        }

        // Check if alias is unique
        if (!$this->isUnique('alias')) {
            $count = 0;
            $currentAlias =  $this->alias;

            while (!$this->isUnique('alias')) {
                $this->alias = $currentAlias . '_' . $count++;
            }
        }

        return parent::check();
    }

    /**
    * Method to store a node in the database table.
    *
    * @param   boolean  $updateNulls  True to update null values as well.
    *
    * @return  boolean  True on success.
    *
    * @link    http://docs.joomla.org/JTableNested/store
    */
    public function store($updateNulls = false)
    {
        $date = Factory::getDate();
        $user = Factory::getUser();

        if ($this->id) {
            // Existing item
            $this->modified_on = $date->toSql();
            $this->modified_by   = $user->id;
        } else {
            if (!(int) $this->created_on) {
                $this->created_on = $date->toSql();
            }

            if (empty($this->created_by)) {
                $this->created_by = $user->id;
            }
        }

        if (empty($this->color)) {
            $this->color = '';
        }

        if (empty($this->params)) {
            $this->params = '';
        }

        return parent::store($updateNulls);
    }

    /**
     * Check if a field is unique
     *
     * @param   string  $field  Name of the field
     *
     * @return bool True if unique
     */
    protected function isUnique($field)
    {
        $db    = $this->_db;
        $query = $db->getQuery(true);

        $query->select($db->quoteName($field))
            ->from($db->quoteName($this->_tbl))
            ->where($db->quoteName($field) . ' = ' . $db->quote($this->$field))
            ->where($db->quoteName('id') . ' <> ' . (int) $this->{$this->_tbl_key})
            ->where($db->quoteName('set_id') . ' = ' . $db->quote($this->set_id));

        $db->setQuery($query);
        $db->execute();

        return ($db->getNumRows() == 0) ? true : false;
    }

    protected function getSetTitle($set_id)
    {
        $db    = $this->_db;
        $query = $db->getQuery(true);

        $query
            ->select($db->quoteName('title'))
            ->from($db->quoteName('#__vmcustomadvanced_sets'))
            ->where($db->quoteName('id') . ' = ' . (int) $set_id);

        $db->setQuery($query);
        $title = $db->loadResult();

        return !empty($title) ? $title : '';
    }

    /**
     * Define a namespaced asset name for inclusion in the #__assets table
     *
     * @return string The asset name
     *
     * @phpcs:disable
     */
    protected function _getAssetName()
    {
        $k = $this->_tbl_key;

        return 'com_vmcustomadvanced.option.' . (int) $this->$k;
    }

    /**
     * Method to return the title to use for the asset table.
     *
     * @return  string
     */
    protected function _getAssetTitle()
    {
        return $this->title;
    }

    /**
     * @phpcs:enable
     *
     * Get the type alias for UCM features
     *
     * @return  string  The alias as described above
     */
    public function getTypeAlias()
    {
        return $this->typeAlias;
    }
}
