<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Table\Table;
use Joomla\Registry\Registry;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;

JLoader::register('VmcustomadvancedTableHistoryInterface', __DIR__ . '/historyinterface.php');
JLoader::register('VMCustomAdvancedHelper', JPATH_ADMINISTRATOR . '/components/com_vmcustomadvanced/helpers/vmcustomadvanced.php');
// phpcs:enable PSR1.Files.SideEffects

/**
 * Set Table class
 */
class VmcustomadvancedTableSet extends Table implements VmcustomadvancedTableHistoryInterface
{
    /**
     * Constructor
     *
     * @param   Joomla\Database\DatabaseDriver   &$db   A database connector object
     */
    public function __construct(&$db)
    {
        if (version_compare(JVERSION, '4.0.0', 'ge')) {
            $this->typeAlias = 'com_vmcustomadvanced.set';
        } else {
            JObserverMapper::addObserverClassToClass('JTableObserverContenthistory', 'VmcustomadvancedTableSet', ['typeAlias' => 'com_vmcustomadvanced.set']);
        }

        parent::__construct('#__vmcustomadvanced_sets', 'id', $db);

        $this->setColumnAlias('published', 'state');
    }

    /**
     * Overloaded bind function to pre-process the params.
     *
     * @param   array  $array   Named array
     * @param   mixed  $ignore  Optional array or list of parameters to ignore
     *
     * @return  null|string  null is operation was satisfactory, otherwise returns an error
     */
    public function bind($array, $ignore = '')
    {
        $app   = Factory::getApplication();
        $input = $app->input;
        $task  = $input->input->get('task');

        if (empty($array['alias'])) {
            $array['alias'] = 'COM_VMCUSTOMADVANCED_SET_TITLE_' . $array['title'];
        }

        $array['alias'] = VMCustomAdvancedHelper::stringLangKeySafe(trim($array['alias']));

        if (isset($array['params']) && is_array($array['params'])) {
            $registry = new Registry();
            $registry->loadArray($array['params']);
            $array['params'] = (string) $registry;
        }

        if (!Factory::getUser()->authorise('core.admin', 'com_vmcustomadvanced.set.' . $array['id'])) {
            $actions         = JAccess::getActionsFromFile(JPATH_ADMINISTRATOR . '/components/com_vmcustomadvanced/access.xml', "/access/section[@name='set']/");
            $default_actions = JAccess::getAssetRules('com_vmcustomadvanced.set.' . $array['id'])->getData();
            $array_jaccess   = [];

            foreach ($actions as $action) {
                if (key_exists($action->name, $default_actions)) {
                    $array_jaccess[$action->name] = $default_actions[$action->name];
                }
            }

            $array['rules'] = $this->JAccessRulestoArray($array_jaccess);
        }

        // Bind the rules for ACL where supported.
        if (isset($array['rules']) && is_array($array['rules'])) {
            $this->setRules($array['rules']);
        }

        return parent::bind($array, $ignore);
    }

    /**
     * Overloaded check function
     *
     * @return bool
     */
    public function check()
    {
        $app   = Factory::getApplication();
        $input = $app->input;
        $task  = $input->input->get('task');

        if ($this->id == 0) {
            $this->ordering = $this->getNextOrder();
        }

        // Check if title is unique
        if (!$this->isUnique('title')) {
            if ($task == 'duplicate' || $task == 'save2copy') {
                $count = 0;
                $currentTitle =  $this->title;

                while (!$this->isUnique('title')) {
                    $this->title = $currentTitle . ' ' . $count++;
                }
            } else {
                $typeName = Text::_('COM_VMCUSTOMADVANCED_SET_TYPE_OPTION_' . (int) $this->type);

                throw new Exception(Text::sprintf('COM_VMCUSTOMADVANCED_ERROR_TITLE_EXISTS', $this->title, $typeName));
            }
        }

        // Check if alias is unique
        if (!$this->isUnique('alias')) {
            $count = 0;
            $currentAlias =  $this->alias;

            while (!$this->isUnique('alias')) {
                $this->alias = $currentAlias . '_' . $count++;
            }
        }

        return parent::check();
    }

    /**
    * Method to store a node in the database table.
    *
    * @param   boolean  $updateNulls  True to update null values as well.
    *
    * @return  boolean  True on success.
    *
    * @link    http://docs.joomla.org/JTableNested/store
    */
    public function store($updateNulls = false)
    {
        $date = Factory::getDate();
        $user = Factory::getUser();

        if ($this->id) {
            // Existing item
            $this->modified_on = $date->toSql();
            $this->modified_by   = $user->id;
        } else {
            if (!(int) $this->created_on) {
                $this->created_on = $date->toSql();
            }

            if (empty($this->created_by)) {
                $this->created_by = $user->id;
            }
        }

        if (empty($this->help_description)) {
            $this->help_description = '';
        }

        if (empty($this->params)) {
            $this->params = '{}';
        }

        if ($this->filter_support) {
            $db = $this->_db;
            $query = $db->getQuery(true);
            $addNew = true;

            if ($this->virtuemart_custom_id > 0) {
                $query->select('*')
                    ->from($db->quoteName('#__virtuemart_customs'))
                    ->where($db->quoteName('virtuemart_custom_id') . ' = ' . (int) $this->virtuemart_custom_id)
                    ->where($db->quoteName('field_type') . ' = ' . $db->quote('S'));

                $db->setQuery($query);
                $custom = $db->loadObject();

                if (!empty($custom)) {
                    $addNew = false;
                }
            }

            if ($addNew) {
                $this->virtuemart_custom_id = 0;
            }

            $data = [];

            $data['virtuemart_custom_id'] = $this->virtuemart_custom_id;
            $data['custom_parent_id'] = 0;
            $data['virtuemart_vendor_id'] = 0;
            $data['custom_jplugin_id'] = 0;
            $data['custom_element'] = '';
            $data['admin_only'] = 0;
            $data['custom_title'] = $this->alias;
            $data['show_title'] = 1;
            $data['custom_tip'] = '';
            $data['custom_value'] = '';
            $data['custom_desc'] = 'This is a hidden custom field automatically managed by VP Advanced Custom Field. Do not delete or edit it.';
            $data['field_type'] = 'S';
            $data['is_list'] = 0;
            $data['is_hidden'] = 1;
            $data['is_cart_attribute'] = 0;
            $data['is_input'] = 0;
            $data['searchable'] = 1;
            $data['layout_pos'] = 'addtocart';
            $data['addEmpty'] = 0;
            $data['selectType'] = 0;
            $data['shared'] = 0;
            $data['published'] = 1;
            $data['transform'] = '';

            $customModel = VmModel::getModel('custom');
            $virtuemart_custom_id = $customModel->store($data);

            if (empty($virtuemart_custom_id)) {
                $this->setError('Failed to create custom field for filtering.');

                return false;
            }

            $this->virtuemart_custom_id = (int) $virtuemart_custom_id;
        }

        return parent::store($updateNulls);
    }

    /**
     * Check if a field is unique
     *
     * @param   string  $field  Name of the field
     *
     * @return bool True if unique
     */
    protected function isUnique($field)
    {
        $db = $this->_db;
        $query = $db->getQuery(true);

        $query->select($db->quoteName($field))
            ->from($db->quoteName($this->_tbl))
            ->where($db->quoteName($field) . ' = ' . $db->quote($this->$field))
            ->where($db->quoteName('id') . ' <> ' . (int) $this->{$this->_tbl_key})
            ->where($db->quoteName('type') . ' = ' . $db->quote($this->type));

        $db->setQuery($query);
        $db->execute();

        return ($db->getNumRows() == 0) ? true : false;
    }

    /**
     * Define a namespaced asset name for inclusion in the #__assets table
     *
     * @return string The asset name
     *
     * @phpcs:disable
     */
    protected function _getAssetName()
    {
        $k = $this->_tbl_key;

        return 'com_vmcustomadvanced.set.' . (int) $this->$k;
    }

    /**
     * Method to return the title to use for the asset table.
     *
     * @return  string
     */
    protected function _getAssetTitle()
    {
        return $this->title;
    }

    /**
     * @phpcs:enable
     *
     * Get the type alias for UCM features
     *
     * @return  string  The alias as described above
     */
    public function getTypeAlias()
    {
        return $this->typeAlias;
    }
}
