/**
 * @package    VP Advanced Custom Field for VirtueMart
 * 
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 */

var VMCustomAdvanced = window.VMCustomAdvanced || {};

(function ($, document, app) {

    app.ajaxPools = {};
    app.cache = {};
    app.validationTimeout = null;
    app.noStockText = null;
    app.hideModalTimeout = null;

    app.bind = function (container) {
        $(container).find('.plg-vmcustom-advanced').each(function () {
            var $form = $(this).closest('form'),
                product_id = $form.find('input[name="virtuemart_product_id[]"]').val();

            app.bindGroup(this);

            if ($form.length && !$form.data('vmcaReady')) {
                app.updateProductPrice($form, product_id);
                app.bindValidation($form[0]);

                $form.data('vmcaReady', true);
            }
        });
    };

    app.bindGroup = function (group) {
        $(group).find('select, input[type="radio"], input[type="checkbox"]').off('change', app.bindField).on('change', app.bindField);

        $(group).find('select').each(function () {
            if (!$(this).data('select2ready')) {
                var $firstOption = $('option:first-child', this),
                    placeholder = null;

                if ($firstOption.length && !$firstOption.val()) {
                    placeholder = $firstOption.text();
                }

                if ($(group).data('nostocktext')) {
                    app.noStockText = $(group).data('nostocktext');
                }

                $(this).select2({
                    placeholder: placeholder,
                    templateResult: app.formatSelect2,
                    templateSelection: app.formatSelect2,
                    escapeMarkup: function (m) {
                        return m;
                    }
                });
            }

            $(this).data('select2ready', true);
        });

        $(group).find('[data-vmcamodal]').each(function () {
            if (!$(this).data('vmcaready')) {
                $(this).on('click', app.showModal).data('vmcaready', true);
            }
        });
    };

    app.formatSelect2 = function (option) {
        if (!option.id) {
            return option.text;
        }

        var html = '',
            hasSample = false;

        if ($(option.element).data('color')) {
            html += '<span class="vmcustomadvanced-select-sample" style="background-color: ' + $(option.element).data('color') + '"></span>';
            hasSample = true;
        } else if ($(option.element).data('image')) {
            if ($(option.element).data('image') == 'NA') {
                html += '<span class="vmcustomadvanced-select-sample-img"><span class="vmcustomadvanced-dummy-img"></span></span>';
            } else {
                html += '<span class="vmcustomadvanced-select-sample-img"><img src="' + $(option.element).data('image') + '" alt="' + option.text + '" /></span>';
            }
            hasSample = true;
        }

        html += '<span class="vmcustomadvanced-select-text">' + option.text + '</span>';

        if ($(option.element).data('price')) {
            html += '<span class="vmcustomadvanced-select-price">' + $(option.element).data('price') + '</span>';
        }

        if (option.disabled && app.noStockText) {
            html += '<span class="vmcustomadvanced-select-nostocktext">' + app.noStockText + '</span>';
        }

        if (hasSample) {
            html = '<span class="vmcustomadvanced-select-item has-sample">' + html + '</span>';
        } else {
            html = '<span class="vmcustomadvanced-select-item">' + html + '</span>';
        }

        return html;
    };

    app.bindValidation = function (form) {
        var $addtocart = $(form).find('button[name="addtocart"], input[name="addtocart"], a[name="addtocart"]');

        if ($addtocart.data('validationTimeout')) {
            clearTimeout($addtocart.data('validationTimeout'));
        }

        $addtocart.data('validationTimeout', setTimeout(function () {
            $addtocart.off('click').off('submit');
            $addtocart.on('click submit', {
                form: form,
                cart: $(form)
            }, app.bindAddtocart);
        }, 100));
    };

    app.bindAddtocart = function (e) {
        e.preventDefault();

        var $form = $(e.data.form),
            $set,
            hasError = false;

        $form.find('.plg-vmcustom-advanced').each(function () {
            var that = this;
            if ($(that).data('required')) {
                $(that).find('.plg-vmcustom-advanced-variant').each(function () {
                    if ($(this).find('input[type="radio"]').length) {
                        if (!$(this).find('input[type="radio"]:checked').length) {
                            hasError = true;
                        }
                    } else if ($(this).find('input[type="checkbox"]').length) {
                        if (!$(this).find('input[type="checkbox"]:checked').length) {
                            hasError = true;
                        }
                    } else {
                        if (!$(this).find('select').val()) {
                            hasError = true;
                        }
                    }

                    if (hasError) {
                        $(this).addClass('plg-vmcustom-advanced-haserror');

                        // Break each loop
                        return false;
                    }
                });
            }
        });

        // Do not allow add to cart
        if (hasError) {
            return false;
        }

        // Proceed to add to cart
        return Virtuemart.addtocart(e);
    };

    app.bindField = function () {
        var $group = $(this).closest('.plg-vmcustom-advanced'),
            $set = $(this).closest('.plg-vmcustom-advanced-variant'),
            $form = $(this).closest('form'),
            product_id = $form.find('input[name="virtuemart_product_id[]"]').val(),
            that = this;

        if (!$group.length || !$set.length) {
            return;
        }

        var custom_id = $group.data('cid'),
            product_id = $group.data('pid'),
            max_level = $group.data('maxlevel'),
            required = $group.data('required'),
            check_stock = $group.data('stockcheck'),
            default_selected = $group.data('defaultselected'),
            level = $set.data('level'),
            set_id = $set.data('setid'),
            option_id = $(that).val(),
            query_url = $group.data('queryurl'),
            selected = {},
            $prev_field,
            prev_set_id,
            prev_option_id,
            query;

        custom_id = app.sanitizeInterger(custom_id);
        product_id = app.sanitizeInterger(product_id);
        level = app.sanitizeInterger(level);
        set_id = app.sanitizeInterger(set_id);
        option_id = app.sanitizeInterger(option_id);

        $set.removeClass('plg-vmcustom-advanced-haserror');

        if (level && max_level && level < max_level) {
            for (i = level + 1; i <= max_level; i++) {
                $group.find('.plg-vmcustom-advanced-variant[data-level="' + i + '"]').remove();
            }
        }

        if (level < max_level) {
            selected[level] = {};
            selected[level]['set_id'] = set_id;
            selected[level]['option_id'] = option_id;

            for (i = level - 1; i > 0; i--) {
                $prevField = $group.find('.plg-vmcustom-advanced-variant[data-level="' + i + '"]');
                prev_set_id = $prevField.data('setid');

                if ($prevField.find('select').length) {
                    prev_option_id = $prevField.find('select').val();
                } else {
                    prev_option_id = $prevField.find('input[type="radio"]:checked').val();
                }

                selected[i] = {};
                selected[i]['set_id'] = app.sanitizeInterger(prev_set_id);
                selected[i]['option_id'] = app.sanitizeInterger(prev_option_id);
            }

            query = {
                custom_id: custom_id,
                product_id: product_id,
                selected: selected,
                level: level + 1
            };

            var key = JSON.stringify(query),
                hash = 'hash' + app.getHashCode(key);

            if (app.cache.hasOwnProperty(hash)) {
                $group.append(app.cache[hash]);
                app.bindGroup($group[0]);
                app.updateProductPrice($form, product_id);
            } else {
                var ajaxPool = $group.data('ajaxPool');

                if (ajaxPool && ajaxPool.readystate != 4) {
                    ajaxPool.abort();
                }

                ajaxPool = $.ajax({
                    type: 'GET',
                    dataType: 'JSON',
                    url: query_url,
                    cache: false,
                    data: query,
                    beforeSend: function () {
                        app.startLoading();
                    },
                    success: function (data) {
                        app.stopLoading();

                        if (!data.result) {
                            if (data.message) {
                                alert(data.message);
                            }
                        } else if (typeof data.html !== typeof undefined) {
                            $group.append(data.html);
                            app.bindGroup($group[0]);
                            app.updateProductPrice($form, product_id);
                            // Save in cache
                            app.cache[hash] = data.html;
                        }
                    },
                    error: function () {
                        console.log('Ajax error.');
                        app.stopLoading();
                    }
                });

                $group.data('ajaxPool', ajaxPool);
            }
        } else {
            app.updateProductPrice($form, product_id);
        }
    };

    app.updateProductPrice = function ($form, product_id) {
        if (typeof Virtuemart !== typeof undefined && typeof Virtuemart.setproducttype == 'function') {
            $form.find('input[data-vpcahidden]').remove();

            $form.find('.plg-vmcustom-advanced[data-multiple="1"]').each(function () {
                var cid = $(this).data('cid'),
                    multipleName = product_id && cid ? 'customProductData[' + product_id + '][' + cid + ']' : null,
                    multipleNameDummy = multipleName ? 'customMultiProductData[' + product_id + '][' + cid + ']' : null,
                    selected = [];

                if (multipleNameDummy && $(this).find('input[type="checkbox"][name="' + multipleNameDummy + '"]:checked').length) {
                    selected = $('input[type="checkbox"][name="' + multipleNameDummy + '"]:checked').map(function (_, el) {
                        return $(el).val();
                    }).get();
                } else if (multipleNameDummy && $(this).find('select[name="' + multipleNameDummy + '"]').length) {
                    selected = $(this).find('select[name="' + multipleNameDummy + '"]').val();
                }

                if (selected && (typeof selected === 'array' || typeof selected === 'object')) {
                    for (i = 0; i < selected.length; i++) {
                        $('<input />').attr('type', 'hidden').attr('name', multipleName + '[' + selected[i] + ']').attr('data-vpcahidden', true).val('1').appendTo($form);
                    }
                }
            });

            $form.find('input[name="view"]').val('productdetails');
            Virtuemart.setproducttype($form, product_id);
            setTimeout(function () {
                $form.find('input[name="view"]').val('cart');
            }, 10);
        } else {
            console.warn('Virtuemart.setproducttype() is not define. Automatic price update will not work.');
        }
    };

    app.getHashCode = function (string) {
        var hash = 0,
            that = new String(string),
            i,
            chr;

        if (that.length === 0) {
            return hash;
        }

        for (i = 0; i < that.length; i++) {
            chr = that.charCodeAt(i);
            hash = ((hash << 5) - hash) + chr;
            hash |= 0; // Convert to 32bit integer
        }

        return hash;
    };

    app.sanitizeInterger = function (val, def) {
        def = typeof def !== typeof undefined ? parseInt(def) : 0;
        def = typeof def === 'number' && Number.isInteger(def) ? def : 0;
        val = parseInt(val);

        if (typeof val === 'number' && Number.isInteger(val)) {
            return val;
        }

        return def;
    };

    app.loadingTimeout = null;

    app.startLoading = function (wrapper) {
        wrapper = typeof wrapper === typeof undefined ? 'body' : wrapper;

        $(wrapper).children('.vmcustomadvanced-loader').remove();

        if (app.loadingTimeout) {
            clearTimeout(app.loadingTimeout);
        }

        $(wrapper).addClass('vmcustomadvanced-loading');

        var $loader = $('<div></div>').addClass('vmcustomadvanced-loader').addClass('vmcustomadvanced-fade'),
            $bg = $('<div></div>').addClass('vmcustomadvanced-loader-bg'),
            $spinner = $('<div></div>').addClass('vmcustomadvanced-loader-spinner');

        $loader.append($bg).append($spinner).appendTo($(wrapper));

        app.loadingTimeout = setTimeout(function () {
            $loader.addClass('vmcustomadvanced-in');
        }, 50);
    };

    app.stopLoading = function (wrapper) {
        wrapper = typeof wrapper === typeof undefined ? 'body' : wrapper;

        if (!$(wrapper).children('.vmcustomadvanced-loader').length) {
            return;
        }

        if (app.loadingTimeout) {
            clearTimeout(app.loadingTimeout);
        }

        $(wrapper).children('.vmcustomadvanced-loader').removeClass('vmcustomadvanced-in');

        app.loadingTimeout = setTimeout(function () {
            $(wrapper).children('.vmcustomadvanced-loader').remove();
            $(wrapper).removeClass('vmcustomadvanced-loading');
        }, 200);
    };

    app.showModal = function (e) {
        e.preventDefault();

        var that = this,
            id = $(that).attr('href'),
            $modal = id ? $(id) : null,
            $modalBg = $('.vmcustomadvanced-modal-bg');

        if (!$modal || !$modal.length) {
            return false;
        }

        var $closeBtn = $modal.find('.vmcustomadvanced-modal-close'),
            $placeholder = $('<div></div>').addClass('vmcustomadvanced-modal-placeholder').attr('data-id', id);

        if (!$closeBtn.data('data-vmcaready')) {
            $closeBtn.on('click', function (a) {
                a.preventDefault();
                return app.hideModal.call(that);
            }).data('data-vmcaready', true);
        }

        if (!$modal.data('data-vmcaready')) {
            $modal.on('click', function (b) {
                b.preventDefault();

                var $modalInner = $('.vmcustomadvanced-modal-inner');

                if (!$modalInner.is(b.target) && $modalInner.has(b.target).length === 0) {
                    return app.hideModal.call(that);
                }
            }).data('data-vmcaready', true);
        }

        $modal.after($placeholder);

        if (app.hideModalTimeout) {
            clearTimeout(app.hideModalTimeout);
        }

        if (!$modalBg.length) {
            $modalBg = $('<div></div>').addClass('vmcustomadvanced-modal-bg');

            $modalBg.appendTo('body');
        }

        $modal.detach().appendTo('body').addClass('vmcustomadvanced-in');
        $modalBg.addClass('vmcustomadvanced-in');

        $(that).data('openedmodal', $modal).data('placeholder', $placeholder).data('modalbg', $modalBg);

        return false;
    };

    app.hideModal = function () {
        var that = this,
            $modal = $(that).data('openedmodal'),
            $placeholder = $(that).data('placeholder'),
            $modalBg = $(that).data('modalbg');

        if (!$modal || !$placeholder) {
            return false;
        }

        $modal.removeClass('vmcustomadvanced-in').detach().insertBefore($placeholder);
        $placeholder.remove();

        if (app.hideModalTimeout) {
            clearTimeout(app.hideModalTimeout);
        }

        if ($modalBg && $modalBg.length) {
            $modalBg.addClass('vmcustomadvanced-modal-fade');

            app.hideModalTimeout = setTimeout(function () {
                $modalBg.removeClass('vmcustomadvanced-modal-fade').removeClass('vmcustomadvanced-in');
            }, 350);
        }

        $(that).data('openedmodal', null).data('placeholder', null).data('modalbg', null);

        return false;
    };

    app.init = function () {
        app.bind(document);

        $('body').on('updateVirtueMartProductDetail', function () {
            app.bind(document);
        });
    };

    $(app.init);

})(jQuery, document, VMCustomAdvanced);