<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;
use Joomla\CMS\Installer\InstallerAdapter;
use Joomla\Database\DatabaseInterface;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * VP Advanced Custom Field Component Script
 *
 * @since   1.0
 */
// phpcs:ignore
class Com_vmcustomadvancedInstallerScript
{
    /**
     * Called on installation
     *
     * @param   InstallerAdapter  $adapter  The object responsible for running this script
     *
     * @return  boolean  True on success
     */
    public function install($adapter)
    {
        try {
            $this->addContentTypes();
        } catch (Exception $e) {
            Factory::getApplication()->enqueueMessage($e->getMessage(), 'error');

            return false;
        }

        return true;
    }

    /**
     * Called on update
     *
     * @param   InstallerAdapter  $adapter  The object responsible for running this script
     *
     * @return  boolean  True on success
     */
    public function update($adapter)
    {
        try {
            $this->addContentTypes();
        } catch (Exception $e) {
            Factory::getApplication()->enqueueMessage($e->getMessage(), 'error');

            return false;
        }

        return true;
    }

    /**
     * Add all required content types.
     */
    private function addContentTypes()
    {
        // Check and add com_vmcustomadvanced.set
        if (!$this->getContentTypeId('com_vmcustomadvanced.set')) {
            $table = [
                'special' => [
                    'dbtable' => '#__vmcustomadvanced_sets',
                    'key'     => 'id',
                    'type'    => 'Set',
                    'prefix'  => 'VmcustomadvancedTable'
                ]
            ];

            $historyOptions = [
                'formFile' => 'administrator/components/com_vmcustomadvanced/models/forms/set.xml',
                'hideFields' => [
                    'checked_out',
                    'checked_out_time',
                    'version',
                    'ordering',
                    'params'
                ],
                'ignoreChanges' => [
                    'ordering',
                    'modified_on',
                    'modified_by',
                    'checked_out',
                    'checked_out_time'
                ],
                'convertToInt' => [
                    'is_button',
                    'filter_support',
                    'virtuemart_custom_id',
                ],
                'displayLookup' => [
                    [
                        'sourceColumn'  => 'created_by',
                        'targetTable'   => '#__users',
                        'targetColumn'  => 'id',
                        'displayColumn' => 'name'
                    ],
                    [
                        'sourceColumn'  => 'modified_by',
                        'targetTable'   => '#__users',
                        'targetColumn'  => 'id',
                        'displayColumn' => 'name'
                    ]
                ]
            ];

            $this->insertContentType('VP Advanced Custom Field Set', 'com_vmcustomadvanced.set', $table, $historyOptions);
        }

        // Check and add com_vmcustomadvanced.option
        if (!$this->getContentTypeId('com_vmcustomadvanced.option')) {
            $table = [
                'special' => [
                    'dbtable' => '#__vmcustomadvanced_options',
                    'key'     => 'id',
                    'type'    => 'Option',
                    'prefix'  => 'VmcustomadvancedTable'
                ]
            ];

            $historyOptions = [
                'formFile' => 'administrator/components/com_vmcustomadvanced/models/forms/option.xml',
                'hideFields' => [
                    'checked_out',
                    'checked_out_time',
                    'version',
                    'ordering'
                ],
                'ignoreChanges' => [
                    'ordering',
                    'modified_on',
                    'modified_by',
                    'checked_out',
                    'checked_out_time'
                ],
                'convertToInt' => [
                    'is_button',
                    'filter_support',
                    'virtuemart_custom_id',
                ],
                'displayLookup' => [
                    [
                        'sourceColumn'  => 'set_id',
                        'targetTable'   => '#__vmcustomadvanced_sets',
                        'targetColumn'  => 'id',
                        'displayColumn' => 'title'
                    ],
                    [
                        'sourceColumn'  => 'created_by',
                        'targetTable'   => '#__users',
                        'targetColumn'  => 'id',
                        'displayColumn' => 'name'
                    ],
                    [
                        'sourceColumn'  => 'modified_by',
                        'targetTable'   => '#__users',
                        'targetColumn'  => 'id',
                        'displayColumn' => 'name'
                    ]
                ]
            ];

            $this->insertContentType('VP Advanced Custom Field Option', 'com_vmcustomadvanced.option', $table, $historyOptions);
        }
    }

    /**
     * Get content type id by alias
     *
     * @param   string   $alias   The alias
     *
     * @return  integer
     */
    private function getContentTypeId($alias)
    {
        $db    = version_compare(JVERSION, '4.0.0', 'ge') ? Factory::getContainer()->get(DatabaseInterface::class) : Factory::getDbo();
        $query = $db->getQuery(true);

        $query->select($db->quoteName('type_id'))
            ->from($db->quoteName('#__content_types'))
            ->where($db->quoteName('type_alias') . ' = ' . $db->quote($alias));

        $db->setQuery($query);

        $id = $db->loadResult();

        // If the row does not exist
        if (empty($id)) {
            return 0;
        }

        return (int) $id;
    }

    /**
     * Insert a content type.
     *
     * @param   string   $title           The title
     * @param   string   $alias           The alias
     * @param   array    $table           The table information
     * @param   array    $historyOptions  The content history options
     *
     * @return  integer  The inserted id
     */
    private function insertContentType(string $title, string $alias, array $table, array $historyOptions)
    {
        $row = new stdClass();

        $row->type_id                 = 0;
        $row->type_title              = $title;
        $row->type_alias              = $alias;
        $row->table                   = json_encode($table);
        $row->rules                   = '';
        $row->field_mappings          = '';
        $row->router                  = '';
        $row->content_history_options = json_encode($historyOptions);

        $db = version_compare(JVERSION, '4.0.0', 'ge') ? Factory::getContainer()->get(DatabaseInterface::class) : Factory::getDbo();

        return $db->insertObject('#__content_types', $row, 'type_id');
    }
}
